from .common import BaseController
from .mpapi.qy import chat as qyChat

class Chat(BaseController):
    def send_some(self, msgType, mediaId, additionalDict={},
            targetIdList=[], partyIdList=[], tagIdList=[],
            agentId=None):
        return self.determine_wrapper(mpMsg.send_some, qyMsg.send_some,
            msgTyp, mediaId, additionalDict,
            targetIdList, partyIdList, tagIdList, agentId)
    def create(self, chatId, name, ownerId, userIdList):
        return self.determine_wrapper(None, qyChat.create,
            chatId, name, ownerId, userIdList)
    def get(self, chatId):
        return self.determine_wrapper(None, qyChat.get,
            chatId)
    def update(self, chatId, opUserId, name=None, ownerId=None,
            addUserIdList=None, delUserIdList=None):
        return self.determine_wrapper(None, qyChat.update,
            chatId, opUserId, name, ownerId, addUserIdList, delUserIdList)
    def quit(self, chatId, opUserId):
        return self.determine_wrapper(None, qyChat.quit,
            opUserId)
    def clear_notify(self, ownerId, chatId=None, userId=None):
        return self.determine_wrapper(None, qyChat.clear_notify,
            ownerId, chatId, userId)
    def send(self, msgType, mediaId, additionalDict={}, senderId=None,
            userId=None, chatId=None):
        return self.determine_wrapper(None, qyChat.send,
            msgTyp, mediaId, additionalDict, senderId, userId, chatId)
    def set_mute(self, muteList=[], cancelList=[]):
        return self.determine_wrapper(None, qyChat.set_mute,
            muteList, cancelList)



from .common import BaseController
from .mpapi.qy import oauth2 as qyOauth

class Oauth2(BaseController):
    def generate_code_url(self, redirectUri, state=None):
        return self.determine_wrapper(None, qyOauth.generate_code_url,
            redirectUri, state)
    def get_user_info(self, code, accessToken=None):
        return self.determine_wrapper(None, qyOauth.get_user_info,
            code)
    def user_id_open_id_switch(self, userId=None, openId=None, agentId=None):
        return self.determine_wrapper(None, qyOauth.user_id_open_id_switch,
            userId, openId, agentId)



import logging

from ..requests import requests
from itchatmp.config import COROUTINE
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry, encode_send_dict

logger = logging.getLogger('itchatmp')

def create_producer(serverUrl):
    def _create(menuDict, agentId=None, accessToken=None):
        data = encode_send_dict(menuDict)
        if data is None: return ReturnValue({'errcode': -10001})
        url = '%s/cgi-bin/menu/create?access_token=%s' % \
            (serverUrl, accessToken)
        if agentId is not None: url += '&agentid=%s' % agentId
        r = requests.post(url, data=data)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    return _create

def get_producer(serverUrl):
    def _get(agentId=None, accessToken=None):
        url = '%s/cgi-bin/menu/get?access_token=%s' % (serverUrl, accessToken)
        if agentId is not None: url += '&agentid=%s' % agentId
        r = requests.get(url)
        def _wrap_result(result):
            result = result.json()
            if 'menu' in result: result['errcode'] = 0
            return ReturnValue(result)
        r._wrap_result = _wrap_result
        return r
    return _get

def delete_producer(serverUrl):
    def _delete(agentId=None, accessToken=None):
        url = '%s/cgi-bin/menu/delete?access_token=%s' % (serverUrl, accessToken)
        if agentId is not None: url += '&agentid=%s' % agentId
        r = requests.get(url)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    return _delete



from requests import get

from itchatmp.config import COMPANY_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry
from ..requests import requests

update_access_token = access_token = get_server_ip = \
    filter_request = lambda x: x



import os, logging
from base64 import b64decode

import tornado

from .content import NORMAL
from .controllers import (
    Application, Chat, Common, CustomerService,
    Menu, Messages, Oauth2, Statistics,
    TemplateMsgs, Users, Utils, Wrapped)
from .components import load_register
from .exceptions import ParameterError
from .models.common import TestStorage

logger = logging.getLogger('itchatmp')

class WechatConfig(object):
    ''' config storing class
     * if copId is set, appId will be ignored
    '''
    def __init__(self, token='', copId='', appId='', appSecret='',
            encryptMode=NORMAL, encodingAesKey=''):
        self.token = token
        self.copId, self.appId, self.appSecret = copId, appId, appSecret
        self.encryptMode = encryptMode
        self.encodingAesKey = encodingAesKey
        try:
            self._encodingAesKey = b64decode(
                self.encodingAesKey.encode('utf8') + b'=')
        except:
            if self.encryptMode == SAFE:
                raise ParameterError('Wrong AES Key format')
            else:
                self._encodingAesKey = ''
    def verify(self):
        return True

class WechatServer(object):
    ''' Wechat server class '''
    def __init__(self, config, atStorage, userStorage,
            filterRequest=False, threadPoolNumber=None):
        # init configurations
        self.config = config
        self.atStorage = atStorage or TestStorage()
        self.userStorage = userStorage
        self.filterRequest = filterRequest
        self.threadPoolNumber = threadPoolNumber or ((None
            if not hasattr(os, 'cpu_count') else os.cpu_count()) or 1) * 5
        try:
            self.ioLoop = tornado.ioloop.IOLoop.current()
        except:
            self.ioLoop = None
        self.isWsgi = True
        self.debug = True
        self._replyFnDict = {}
        # init apis
        self.application = Application(self)
        self.chat = Chat(self)
        self.common = Common(self)
        self.customerservice = CustomerService(self)
        self.menu = Menu(self)
        self.messages = Messages(self)
        self.oauth2 = Oauth2(self)
        self.statistics = Statistics(self)
        self.templatemsgs = TemplateMsgs(self)
        self.users = Users(self)
        self.utils = Utils(self)
        self.wrapped = Wrapped(self)
    def update_config(self, config=None, atStorage=None, userStorage=None,
            filterRequest=None, threadPoolNumber=None):
        ''' it is defined in components/register '''
        raise NotImplementedError()
    def run(self, isWsgi=False, debug=True, port=80):
        ''' it is defined in components/register '''
        raise NotImplementedError()
    def msg_register(self, msgType):
        ''' it is defined in components/register '''
        raise NotImplementedError()
    def upload(self, fileType, fileDir, additionalDict={}, permanent=False):
        return self.messages.upload(fileType, fileDir, additionalDict, permanent)
    def send(self, msg, toUserName, mediaId=None):
        return self.wrapped.send(msg, toUserName, mediaId)
    def filter_request(self, request):
        ''' this is not open for calling '''
        return self.common.filter_request(request)
    def access_token(self, fn):
        return self.common.access_token(fn)
    def clear_quota(self):
        return self.common.clear_quota()

load_register(WechatServer)



from .common import BaseController
from .mpapi.mp import statistics as mpStat

'''
yiw"lpfuviwpyiw2jfu;viwp3k2d3j
'''

class Statistics(BaseController):
    def user_summary(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_summary, None, *args)
    def user_cumulate(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_cumulate, None, *args)
    def article_summary(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.article_summary, None, *args)
    def article_total(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.article_total, None, *args)
    def user_read(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_read, None, *args)
    def user_read_hour(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_read_hour, None, *args)
    def user_share(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_share, None, *args)
    def user_share_hour(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.user_share_hour, None, *args)
    def upstream_msg(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg, None, *args)
    def upstream_msg_hour(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_hour, None, *args)
    def upstream_msg_week(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_week, None, *args)
    def upstream_msg_month(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_month, None, *args)
    def upstream_msg_dist(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_dist, None, *args)
    def upstream_msg_dist_week(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_dist_week, None, *args)
    def upstream_msg_dist_month(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.upstream_msg_dist_month, None, *args)
    def interface_summary(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.interface_summary, None, *args)
    def interface_summary_hour(self, startTime, timeSection=None):
        args = (startTime,) if timeSection is None else (startTime, timeSection)
        return self.determine_wrapper(mpStat.interface_summary_hour, None, *args)



''' This package is for mass texting in wechat company platform
 1. What can we send?
    - TEXT, IMAGE, VOICE, VIDEO, FILE, NEWS

 2. How to send them?
    - we use send_some / send_all method
        `send_some(targetIdList, msgType, mediaId, additionalDict
            targetIdList, partyIdList, tagIdList, agentId)`
    - for msg like text, just pass content as msgId
    - for files like image, voice, video, we need to upload them first
        `upload(fileType, openedFile, additionalDict, permanent)`
    - for news, you need to form them first and upload to get msgId
        `create_news(newsDict, permanent)`
      for images used in news, you need to turn them into url first
        `get_image_url(openedFile)`
    - SPECIAL WARNING: video is a little bit **STRANGE**
      when uploading or sending, you need to pass additionalDict to method
        `{"title" :VIDEO_TITLE, "introduction" :INTRODUCTION}`

 3. I alse listed API list for you:
   - SENDING
     send_some
     send_all
   - TEMP MATERIAL MANAGING
     upload
     download
   - PERMENENT MATERIAL MANAGING
     get_material
     delete_material
     get_materialcount
     batchget_material
   - FORM NEWS
     create_news
     update_news
     get_image_url
'''
import logging, json

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import COMPANY_URL
from itchatmp.content import (FILE,
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def send_some(msgType, mediaId, additionalDict={},
        targetIdList=[], partyIdList=[], tagIdList=[],
        agentId=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict:
        return msgDict
    elif not (targetIdList or partyIdList or tagIdList):
        return ReturnValue({'errcode': 40130, 'errmsg':
            'there must be one filled list'})
    elif agentId is None: 
        return ReturnValue({'errcode': -10003, 'errmsg':
            'agentId must be set'})
    msgDict['touser']  = '|'.join(targetIdList)
    msgDict['toparty'] = '|'.join(partyIdList)
    msgDict['totag']   = '|'.join(tagIdList)
    msgDict['agentid'] = agentId
    msgDict['safe']    = 0
    data = encode_send_dict(msgDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/message/send?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def send_all(msgType, mediaId, additionalDict={}, tagId=None,
        agentId=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict:
        return msgDict
    elif agentId is None: 
        return ReturnValue({'errcode': -10003, 'errmsg':
            'agentId must be set'})
    msgDict['touser']  = '@all'
    msgDict['toparty'] = '@all'
    msgDict['totag']   = '@all'
    msgDict['agentid'] = agentId
    msgDict['safe']    = 0
    data = encode_send_dict(msgDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/message/send?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

def __form_send_dict(msgType, mediaId, additionalDict):
    if not msgType in (TEXT, IMAGE, VOICE, VIDEO, FILE, NEWS):
        return ReturnValue({'errcode': 40004,})
    return {
        TEXT: {'text': {'content': mediaId}, 'msgtype': 'text'},
        IMAGE: {'image': {'media_id': mediaId}, 'msgtype': 'image'},
        VOICE: {'voice': {'media_id': mediaId}, 'msgtype': 'voice'},
        VIDEO: {'video':{'media_id': mediaId,
            'title': additionalDict.get('title', ''),
            'description': additionalDict.get('introduction', '')},
            'msgtype': 'video'},
        FILE: {'file': {'media_id': mediaId}, 'msgtype': 'file'},
        NEWS: {'mpnews':{'media_id': mediaId}, 'msgtype': 'mpnews'},
        }[msgType]

@access_token
def upload(fileType, fileDir, additionalDict={}, permanent=False, accessToken=None):
    if not fileType in (IMAGE, VOICE, VIDEO, FILE):
        return ReturnValue({'errcode': 40004,})
    # elif fileType == VIDEO and not ('title' in additionalDict
    #         and 'introduction' in additionalDict):
    #     return ReturnValue({'errcode': -10001, 'errmsg': 
    #         'additionalDict for type VIDEO should be: ' + 
    #         '{"title" :VIDEO_TITLE, "introduction" :INTRODUCTION}'})
    if permanent:
        url = '%s/cgi-bin/material/add_material?access_token=%s&type=%s'
    else:
        url = '%s/cgi-bin/media/upload?access_token=%s&type=%s' 
    # if fileType == VIDEO:
    #     files = {
    #         'description': (None, encode_send_dict(additionalDict),
    #             'application/json'),
    #         'file': ('tmp.mp4', openedFile, 'video/mp4') }
    # else:
    if hasattr(fileDir, 'fileno'):
        files = {'file': fileDir}
        r = requests.post(url % (COMPANY_URL, accessToken, fileType),
            files=files).json()
    else:
        with open(fileDir, 'rb') as f:
            files = {'file': f}
            r = requests.post(url % (COMPANY_URL, accessToken, fileType),
                files=files).json()
    if 'media_id' in r:
        r['errcode'] = 0
    return ReturnValue(r)

@access_token
def download(mediaId, accessToken=None):
    params = {
        'access_token': accessToken,
        'media_id': mediaId, }
    r = requests.get('%s/cgi-bin/media/get' % COMPANY_URL, stream=True)
    if 'application/json' in r.headers['Content-Type']:
        return ReturnValue(r.json())
    else:
        tempStorage = io.BytesIO()
        for block in r.iter_content(1024):
            tempStorage.write(block)
        return ReturnValue({'file': tempStorage, 'errcode': 0})

@access_token
def get_material(mediaId, accessToken=None):
    params = {
        'media_id': mediaId,
        'access_token': accessToken, }
    r = requests.get('%s/cgi-bin/material/get' % COMPANY_URL,
        params=params, stream=True)
    if 'application/json' in r.headers['Content-Type']:
        j = r.json()
        if 'type' in j: j['errcode'] = 0
        return ReturnValue(j)
    else:
        tempStorage = io.BytesIO()
        for block in r.iter_content(1024):
            tempStorage.write(block)
        return ReturnValue({'file': tempStorage, 'errcode': 0})
    return ReturnValue(r)

@access_token
def delete_material(mediaId, accessToken=None):
    params = {
        'media_id': mediaId,
        'access_token': accessToken, }
    r = requests.get('%s/cgi-bin/material/del' % COMPANY_URL,
        params=params).json()
    return ReturnValue(r)

@access_token
def get_material_count(accessToken=None):
    r = requests.get('%s/cgi-bin/material/get_count?access_token=%s'
        % (COMPANY_URL, accessToken)).json()
    return ReturnValue(r)

@access_token
def batchget_material(fileType, offset=0, count=20, accessToken=None):
    if not fileType in (NEWS, IMAGE, VOICE, VIDEO, FILE):
        return ReturnValue({'errcode': 40004,})
    if fileType == NEWS: fileType = 'mpnews'
    if 50 < count: count = 50
    data = {'type': fileType,
        'offset': offset,
        'count': count, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/material/batchget?access_token=%s'
        % (COMPANY_URL, accessToken), data=data).json()
    if 'itemlist' in r: r['errcode'] = 0
    return ReturnValue(r)

@access_token
def create_news(newsDict, permanent=False, accessToken=None):
    if permanent:
        url = '%s/cgi-bin/material/add_mpnews?access_token=%s'
    else:
        return ReturnValue({'errcode': -10003, 'errmsg':
            'All news must be permanent for company platform'})
    data = encode_send_dict(newsDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post(url % (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def update_news(mediaId, newsDict, index=0, accessToken=None):
    data = {
        'media_id': mediaId,
        'mpnews': newsDict, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/material/update_mpnews?access_token=%s' %
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def get_image_url(openedFile, accessToken=None):
    r = requests.post('%s/cgi-bin/media/uploadimg?access_token=%s' % 
        (COMPANY_URL, accessToken), files={'file': openedFile}).json()
    if 'url' in r: r['errcode'] = 0
    return ReturnValue(r)



VERSION = '0.0.17'

SERVER_URL = 'https://api.weixin.qq.com'
COMPANY_URL = 'https://qyapi.weixin.qq.com'

SERVER_WAIT_TIME = 4.5

GREETING_WORDS = 'Greeting from itchatmp!'

try:
    import itchatmphttp
    COROUTINE = True
except ImportError:
    COROUTINE = False



from itchatmp.content import (TEXT,
    IMAGE, VOICE, VIDEO, MUSIC,
    NEWS, TRANSFER, ENCRYPT)

templateDict = {
    TEXT:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[text]]></MsgType>
    <Content><![CDATA[{Content}]]></Content>
    </xml>
    ''',
    IMAGE:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[image]]></MsgType>
    <Image>
    <MediaId><![CDATA[{MediaId}]]></MediaId>
    </Image>
    </xml>
    ''',
    VOICE:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[voice]]></MsgType>
    <Voice>
    <MediaId><![CDATA[{MediaId}]]></MediaId>
    </Voice>
    </xml>
    ''',
    VIDEO:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[video]]></MsgType>
    <Video>
    <MediaId><![CDATA[{MediaId}]]></MediaId>
    <Title><![CDATA[{Title}]]></Title>
    <Description><![CDATA[{Description}]]></Description>
    </Video>
    </xml>
    ''',
    MUSIC:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[music]]></MsgType>
    <Music>
    <Title><![CDATA[{Title}]]></Title>
    <Description><![CDATA[{Description}]]></Description>
    <MusicUrl><![CDATA[{MusicUrl}]]></MusicUrl>
    <HQMusicUrl><![CDATA[{HQMusicUrl}]]></HQMusicUrl>
    <ThumbMediaId><![CDATA[{ThumbMediaId}]]></ThumbMediaId>
    </Music>
    </xml>
    ''',
    NEWS:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[news]]></MsgType>
    <ArticleCount>{ArticleCount}</ArticleCount>
    <Articles>{Articles}</Articles>
    </xml>
    ''',
    'article':
    u'''
    <item>
    <Title><![CDATA[{Title}]]></Title>
    <Description><![CDATA[{Description}]]></Description>
    <PicUrl><![CDATA[{PicUrl}]]></PicUrl>
    <Url><![CDATA[{Url}]]></Url>
    </item>
    ''',
    TRANSFER:
    u'''
    <xml>
    <ToUserName><![CDATA[{ToUserName}]]></ToUserName>
    <FromUserName><![CDATA[{FromUserName}]]></FromUserName>
    <CreateTime>{CreateTime}</CreateTime>
    <MsgType><![CDATA[transfer_customer_service]]></MsgType>
    </xml>
    ''',
    ENCRYPT: 
    u'''
    <xml>
    <Encrypt><![CDATA[{Encrypt}]]></Encrypt>
    <MsgSignature><![CDATA[{MsgSignature}]]></MsgSignature>
    <TimeStamp>{TimeStamp}</TimeStamp>
    <Nonce><![CDATA[{Nonce}]]></Nonce>
    </xml>
    ''',
}
for k, v in templateDict.items():
    if isinstance(templateDict[k], type(u'')):
        templateDict[k] = v.replace(' ', '').replace('\n', '')
    else:
        for i, item in enumerate(templateDict[k]):
            templateDict[k][i] = item.replace(' ', '').replace('\n', '')

def get_template(msgType):
    return templateDict.get(msgType, '')



import pickle, logging, sys

logger = logging.getLogger('itchatmp')

class AccessTokenStorage(object):
    def get_access_token(self):
        raise NotImplementedError()
    def store_access_token(self, accessToken, expireTime):
        raise NotImplementedError()
    def get_server_list(self):
        raise NotImplementedError()
    def store_server_list(self, serverList, fetchTime):
        raise NotImplementedError()

class TestStorage(AccessTokenStorage):
    ''' storage for test use
        {
            'accessToken': ('', 0),
            'serverList': ([], 0),
        }
    '''
    def __init__(self):
        try:
            with open('storage.pkl', 'rb') as f:
                self.__storageDict = pickle.load(f)
        except:
            logger.debug('storage not found')
            self.__storageDict = {}
    def __store_locally(self):
        try:
            with open('storage.pkl', 'wb') as f:
                pickle.dump(self.__storageDict, f)
        except:
            pass
    def get_access_token(self):
        return self.__storageDict.get('accessToken', ('', 0))
    def store_access_token(self, accessToken, expireTime):
        self.__storageDict['accessToken'] = (accessToken, expireTime)
        self.__store_locally()
        logger.debug('Access token updated')
    def get_server_list(self):
        return self.__storageDict.get('serverList', ([], 0))
    def store_server_list(self, serverList, fetchTime):
        self.__storageDict['serverList'] = (serverList, fetchTime)
        self.__store_locally()
        logger.debug('Server list updated')

class MemCacheStorage(AccessTokenStorage):
    ''' storage for memcache
        'accessToken': ('', 0),
        'serverList': ([], 0),
    '''
    def __init__(self):
        try:
            import pylibmc
        except ImportError:
            logger.info('pylibmc is not installed')
            sys.exit()
        self.__storage = pylibmc.Client()
    def get_access_token(self):
        return self.__storage.get('accessToken') or ('', 0)
    def store_access_token(self, accessToken, expireTime):
        self.__storage.set('accessToken', (accessToken, expireTime))
        logger.debug('Access token updated')
    def get_server_list(self):
        return self.__storage.get('serverList') or ([], 0)
    def store_server_list(self, serverList, fetchTime):
        self.__storage.set('serverList', (serverList, fetchTime))
        logger.debug('Server list updated')



from .server import WechatConfig, WechatServer
from .models.common import TestStorage
from .config import VERSION
from .controllers.envtest import env_test

from .log import set_logging

__version__ = VERSION

instanceList = []

def new_instance():
    newInstance = WechatServer(None, None, None)
    instanceList.append(newInstance)
    return newInstance

originInstance = new_instance()

# I really want to use sys.modules[__name__] = originInstance
# but it makes auto-fill a real mess, so forgive me for my following **
# actually it toke me less than 30 seconds, god bless Uganda

set_logging   = set_logging

update_config = originInstance.update_config
run           = originInstance.run
msg_register  = originInstance.msg_register
send          = originInstance.send
access_token  = originInstance.access_token
clear_quota   = originInstance.clear_quota

application = originInstance.application
chat = originInstance.chat
common = originInstance.common
customerservice = originInstance.customerservice
menu = originInstance.menu
messages = originInstance.messages
oauth2 = originInstance.oauth2
statistics = originInstance.statistics
templatemsgs = originInstance.templatemsgs
users = originInstance.users
utils = originInstance.utils
wrapped = originInstance.wrapped



class BaseException(Exception):
    ''' exception base '''
    pass

class ParameterError(BaseException):
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message

class EnvironmentError(BaseException):
    def __init__(self, message=''):
        self.message = message
    def __str__(self):
        return self.message



from .application import Application
from .chat import Chat
from .common import Common
from .customerservice import CustomerService
from .menu import Menu
from .messages import Messages
from .oauth2 import Oauth2
from .statistics import Statistics
from .templatemsgs import TemplateMsgs
from .users import Users
from .utils import Utils
from .wrapped import Wrapped



from tornado.concurrent import Future

from itchatmp.utils import CoreMixin
from .mpapi.mp import common as mpCommon
from .mpapi.qy import common as qyCommon

class BaseController(CoreMixin):
    def determine_wrapper(self, mpFn=None, copFn=None, *args, **kwargs):
        tokenFn = self.core.common.access_token
        if 'needToken' in kwargs:
            if not kwargs['needToken']:
                tokenFn = lambda fn: fn
            del kwargs['needToken']
        if self.core.config.copId != '':
            if copFn is None:
                raise AttributeError('No such method for company platform')
            else:
                return tokenFn(copFn)(*args, **kwargs)
        elif self.core.config.appId != '':
            if mpFn is None:
                raise AttributeError('No such method for massive platform')
            else:
                return tokenFn(mpFn)(*args, **kwargs)
        else:
            raise AttributeError('You must specific appId or copId before use this method')

class Common(BaseController):
    def __init__(self, core):
        super(Common, self).__init__(core)
        self.mpToken = mpCommon.TokenClass(core)
        self.mpServerList = mpCommon.ServerListClass(core, self.mpToken)
    def update_access_token(self):
        return self.determine_wrapper(self.mpToken.update_access_token,
            qyCommon.update_access_token, needToken=False)
    def access_token(self, fn):
        return self.determine_wrapper(self.mpToken.access_token, 
            qyCommon.access_token, fn,
            needToken=False)
    def get_server_ip(self):
        return self.determine_wrapper(self.mpServerList.get_server_ip, 
            qyCommon.get_server_ip, needToken=False)
    def filter_request(self, request):
        return self.determine_wrapper(self.mpServerList.filter_request, 
            qyCommon.filter_request, request,
            needToken=False)
    def clear_quota(self):
        return self.determine_wrapper(mpCommon.clear_quota, None)



import re, logging

from tornado import gen

from itchatmp.config import COROUTINE
from itchatmp.content import (MUSIC,
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue
from itchatmp.views import reply_msg_format

logger = logging.getLogger('itchatmp')

def send(core, msg, toUserId, mediaId=None, accessToken=None):
    ''' This is a method for sending messages to a specific user
     1. How this works?
        - if the user has contacted mp in 48 hours, use customerservice.send
        - if not, use messages.preview
        - MUSIC, which doesn't support preview will return 45015 (out of 48h)
     2. What should be passed into msg
        - it can be a string (for encoding other than ascii, unicode should be used)
          - it should start with things like @img@
          - then it should follow mediaId or fileDir (for text, it's content)
          - it supports: img, voc, vid, txt, nws, cad (msc is not supported)
        - it can be a dict (for encoding other than ascii, unicode should be used)
          - value of key "MsgType" should be the msgType
          - value of key "MediaId" should be mediaId (for text, value is content)
          - for VIDEO, there need to be two more keys:
            - Title, Introduction
          - for MUSIC, there need to be three more keys:
            - MusicUrl, HqMusicUrl, ThumbMediaId
          - it supports all types: img, voc, vid, txt, nws, cad, msc
    '''
    msg = reply_msg_format(msg) # format string into dict
    # filter unexpected messages
    if 'MsgType' in msg:
        msgType = msg['MsgType']
    else:
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'value of key "MsgType" should be a valid message type'})
    if msgType not in (IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC):
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'send supports: IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC'})
    if COROUTINE:
        @gen.coroutine
        def _send(mediaId):
            mediaId = mediaId or msg.get('MediaId', '')
            if 'FileDir' in msg and msgType != TEXT and not mediaId:
                r = yield core.messages.upload(msgType, msg['FileDir'], msg, msg.get('Permanent', False))
                if not r:
                    raise gen.Return(r)
                mediaId = r['media_id']
            r = yield core.customerservice.send(msgType, mediaId, additionalDict=msg, toUserId=toUserId)
            r['preview'] = False
            if not r:
                if r['errcode'] != 45015 or msgType == MUSIC:
                    raise gen.Return(r)
                r = yield core.messages.preview(msgType, mediaId, additionalDict=msg, toUserId=toUserId)
                r['preview'] = True
            raise gen.Return(r)
        return _send(mediaId)
    else:
        mediaId = mediaId or msg.get('MediaId', '')
        if 'FileDir' in msg and msgType != TEXT and not mediaId:
            r = core.messages.upload(msgType, msg['FileDir'], msg, msg.get('Permanent', False))
            if not r:
                return r
            mediaId = r['media_id']
        r = core.customerservice.send(msgType, mediaId, additionalDict=msg, toUserId=toUserId)
        r['preview'] = False
        if not r:
            if r['errcode'] != 45015 or msgType == MUSIC:
                return r
            r = core.messages.preview(msgType, mediaId, additionalDict=msg, toUserId=toUserId)
            r['preview'] = True
        return r



try:
    from urllib import quote_plus as quote
except ImportError:
    from urllib.parse import quote_plus as quote
import time
from itchatmp.config import COMPANY_URL
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry, encode_send_dict
from ..requests import requests
from .common import access_token

# __server
def generate_code_url(redirectUri, state=None):
    ''' generate redirect url for visiting with code
     * you don't need to urlencode redirectUri
    '''
    return ('https://open.weixin.qq.com/connect/oauth2/authorize?' + 
        'appid=%s&redirect_uri=%s&response_type=code&scope=snsapi_base' +
        '&state=%s#wechat_redirect') % \
        ('__server.config.copId', quote(redirectUri), quote((state or str(int(time.time())))))

@access_token
def get_user_info(code, accessToken=None):
    params = {
        'access_token': accessToken,
        'code': code, }
    r = requests.get('%s/cgi-bin/user/getuserinfo' % COMPANY_URL,
        params=params).json()
    if 'DeviceId' in r: r['errcode'] = 0
    return ReturnValue(r)

@access_token
def user_id_open_id_switch(userId=None, openId=None, agentId=None, accessToken=None):
    data = {}
    if userId:
        data['userid'] = userId
        if agentId: data['agentid'] = agentId
        url = '%s/cgi-bin/user/convert_to_openid?access_token=' + accessToken
    elif openId:
        data['openid'] = openId
        url = '%s/cgi-bin/user/convert_to_userid?access_token=' + accessToken
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post(url % COMPANY_URL, data=data).json()
    return ReturnValue(r)

@access_token
def get_login_info(code, accessToken=None):
    data = {'auth_code': code, }
    r = requests.post('%s/cgi-bin/service/get_login_info?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    if 'usertype' in r: r['errcode'] = 0
    return ReturnValue(r)



import socket

from itchatmp.exceptions import EnvironmentError

PYCRYPTO_WARNING = '''
pycrypto is not installed correctly.
* You may choose to install cryptography instead
* If you are using OSX or linux
    - you may uninstall and reinstall it use sudo
* If you are using Windows
    - you may uninstall and reinstall it in admin
    - you may change %PYTHONPATH%\\site-packages\\crypto
        to %PYTHONPATH%\\site-packages\\Crypto
* If they didn't work for you
    - contact me through i7meavnktqegm1b@qq.com
    - or through github.com/littlecodersh/
    - or report an issue \
'''
PORT_WARNING = 'port 80 is in use.'

def env_test(port):
    try:
        from Crypto.Cipher import AES
    except ImportError:
        try:
            from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
            from cryptography.hazmat.backends import default_backend
        except ImportError:
            raise EnvironmentError(PYCRYPTO_WARNING)
    try:
        s = socket.socket()
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        s.bind(('127.0.0.1', port))
        s.listen(1)
        s.close()
    except:
        raise EnvironmentError(PORT_WARNING)



import time, logging, re, os, json
import traceback
try:
    import lxml.etree as ET
except ImportError:
    import xml.etree.ElementTree as ET

from itchatmp.content import (ENCRYPT, MUSIC,
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.exceptions import ParameterError
from itchatmp.returnvalues import ReturnValue
from .templates import get_template

logger = logging.getLogger('itchatmp')

def deconstruct_msg(msg):
    ''' deconstruct xml msg from string to dict
     * use lxml to save time
     * if deconstruct failed will return an empty dict
    '''
    r = {}
    def _get_dict(msg, d):
        for i in msg:
            if i.text is None:
                d[i.tag] = {}
                _get_dict(i, d[i.tag])
            else:
                d[i.tag] = i.text
    try:
        _get_dict(ET.fromstring(msg), r)
    except:
        logger.debug(traceback.format_exc())
    if 'Encrypt' in r:
        r['MsgType'] = r.get('MsgType') or ENCRYPT
    return r

def reply_msg_format(msg):
    ''' turns string reply to reply dict
     * if format failed, it will return a ReturnValue equals to False
     * if succeeded, it will return a dict equals to string
     * string content will be filled in MediaId key
    '''
    if isinstance(msg, dict):
        r = msg
    elif hasattr(msg, 'capitalize'):
        msgType, content = msg[1:4], msg[5:]
        r = {}
        if not re.match('@[a-z]{3}@', msg[:5]):
            r['MsgType'] = TEXT
            r['MediaId'] = msg
        elif msgType == 'msc':
            return ReturnValue({'errcode': -10003, 'errmsg': 
                'msg for type MUSIC should be: {"MsgType": MUSIC, ' + 
                '"musicurl" :MUSICURL, "hqmusicurl" :HQMUSICURL, ' +
                '"thumb_media_id": MEDIA_ID}'})
        elif msgType not in ('img', 'voc', 'vid', 'txt', 'nws', 'cad'):
            return ReturnValue({'errcode': -10003, 'errmsg': 
                'send supports: img, voc, vid, txt, nws, cad'})
        elif msgType == 'txt':
            r['MsgType'] = TEXT
            r['MediaId'] = content
        elif msgType in ('nws', 'cad'):
            r['MsgType'] = NEWS if msgType == 'nws' else CARD
            try:
                r.update(json.loads(content))
            except:
                logger.warning('content of %s is not a valid json' % r['MsgType'])
        else:
            r['MsgType'] = {'img': IMAGE, 'voc': VOICE, 'vid': VIDEO}[msgType]
            if os.path.isfile(content):
                r['FileDir'] = content
            else:
                r['MediaId'] = content
    else:
        r = ReturnValue({'errcode': -10003, 'errmsg': 
            'msg should be string or dict'})
    if 'Content' in r and not 'MediaId' in r and r.get('MsgType') == TEXT:
        r['MediaId'] = r['Content']
    return r

def construct_msg(replyDict):
    ''' construct xml msg from dict to string
     * if deconstruct failed will return an empty string
    '''
    def _render(template, replyDict):
        try:
            if 'CreateTime' not in replyDict:
                replyDict['CreateTime'] = int(time.time())
            return template.format(**replyDict)
        except KeyError as e:
            logger.debug('Missing message element "%s"' % e.args)
        except UnicodeDecodeError as e:
            logger.debug('All non-ascii values should be unicode like: u"value"')
        except:
            logger.debug(
                'Wrong format of reply message: ' + str(replyDict))
            logger.debug(traceback.format_exc())
        return ''
    def _fill_key(d, k):
        d[k] = d.get(k, '')
    if replyDict['MsgType'] == TEXT:
        if 'MediaId' in replyDict and 'Content' not in replyDict:
            replyDict['Content'] = replyDict['MediaId']
    elif replyDict['MsgType'] == VIDEO:
        for k in ('Title', 'Description'):
            _fill_key(replyDict, k)
    elif replyDict['MsgType'] == MUSIC:
        for k in ('Title', 'Description', 'MusicURL', 'HQMusicUrl'):
            _fill_key(replyDict, k)
    elif replyDict['MsgType'] == NEWS:
        for k in ('Title', 'Description', 'PicUrl', 'Url'):
            _fill_key(replyDict, k)
        if not 0 < len(replyDict.get('Articles',{})) < 10:
            raise ParameterError(
                'A news must have 1-9 articles')
        replyDict['ArticleCount'] = len(replyDict['Articles'])
        replyDict['Articles'] = ''.join(
            [_render(get_template('article'), article)
            for article in replyDict['Articles']])
    return _render(get_template(replyDict['MsgType']), replyDict)



from .common import BaseController
from .mpapi.mp import utils as mpUtils

class Utils(BaseController):
    def create_qrcode(self, sceneData, expire=2592000):
        return self.determine_wrapper(mpUtils.create_qrcode, None,
            sceneData, expire)
    def download_qrcode(self, ticket):
        return self.determine_wrapper(mpUtils.download_qrcode, None,
            ticket)
    def long_url_to_short(self, url):
        return self.determine_wrapper(mpUtils.long_url_to_short, None,
            url)



from .common import BaseController
from .mpapi.mp import users as mpUsers
from .mpapi.qy import users as qyUsers

class Users(BaseController):
    def create_tag(self, name, id_=None):
        return self.determine_wrapper(mpUsers.create_tag, qyUsers.create_tag,
            name, id_)
    def get_tags(self):
        return self.determine_wrapper(mpUsers.get_tags, qyUsers.get_tags)
    def update_tag(self, id_, name):
        return self.determine_wrapper(mpUsers.update_tag, qyUsers.update_tag,
            id_, name)
    def delete_tag(self, id_):
        return self.determine_wrapper(mpUsers.delete_tag, qyUsers.delete_tag,
            id_)
    def get_users_of_tag(self, id_, nextOpenId=''):
        return self.determine_wrapper(mpUsers.get_users_of_tag, qyUsers.get_users_of_tag,
            id_, nextOpenId)
    def add_users_into_tag(self, id_, userIdList=None, partyList=None):
        return self.determine_wrapper(
            mpUsers.add_users_into_tag, qyUsers.add_users_into_tag,
            id_, userIdList, partyList)
    def delete_users_of_tag(self, id_, userIdList=None, partyList=None):
        return self.determine_wrapper(
            mpUsers.delete_users_of_tag, qyUsers.delete_users_of_tag,
            id_, userIdList, partyList)
    def get_tags_of_user(self, userId):
        return self.determine_wrapper(mpUsers.get_tags_of_user, None,
            userId)
    def set_alias(self, userId, alias):
        return self.determine_wrapper(mpUsers.set_alias, None,
            userId, alias)
    def get_user_info(self, userId):
        return self.determine_wrapper(mpUsers.get_user_info, qyUsers.get_user_info,
            userId)
    def get_users(self, nextOpenId='', departmentId=None, fetchChild=False, status=4):
        return self.determine_wrapper(mpUsers.get_users, qyUsers.get_users,
            nextOpenId, departmentId, fetchChild, status)
    def get_detailed_users(self, nextOpenId='', departmentId=None,
            fetchChild=False, status=4):
        return self.determine_wrapper(None, qyUsers.get_detailed_users,
            nextOpenId, departmentId, fetchChild, status)
    def get_blacklist(self, beginOpenId=''):
        return self.determine_wrapper(mpUsers.create_tag, None,
            beginOpenId)
    def add_users_into_blacklist(self, userId):
        return self.determine_wrapper(mpUsers.add_users_into_blacklist, None,
            userId)
    def delete_users_of_blacklist(self, userId):
        return self.determine_wrapper(mpUsers.delete_users_of_blacklist, None,
            userId)
    def authorize_user(self, userId):
        return self.determine_wrapper(None, qyUsers.authorize_user,
            userId)
    def create_department(self, name, parentId=1, order=None, id_=None):
        return self.determine_wrapper(None, qyUsers.create_department,
            name, parentId, order, id_)
    def update_department(self, id_, name=None, parentId=None, order=None):
        return self.determine_wrapper(None, qyUsers.update_department,
            id_, name, parentId, order)
    def delete_department(self, id_):
        return self.determine_wrapper(None, qyUsers.delete_department,
            id_)
    def get_departments(self, parentId):
        return self.determine_wrapper(None, qyUsers.get_departments,
            parentId)
    def create_user(self, userId, name, departmentIdList,
            position=None, mobile=None, gender=None, email=None,
            weixinId=None, headImgId=None, extAttr=None):
        return self.determine_wrapper(None, qyUsers.create_user,
            userId, name, departmentIdList, position,
            mobile, gender, email, weixinId, headImgId, extAttr)
    def update_user(self, userId, name=None, departmentIdList=None,
            position=None, mobile=None, gender=None, email=None,
            weixinId=None, headImgId=None, extAttr=None):
        return self.determine_wrapper(None, qyUsers.update_user,
            userId, name, departmentIdList, position,
            mobile, gender, email, weixinId, headImgId, extAttr)
    def delete_users(self, userId):
        return self.determine_wrapper(None, qyUsers.delete_users,
            userId)
    def upload_contract(self, csvMediaId, callbackUrl, method='sync'):
        return self.determine_wrapper(None, qyUsers.upload_contract,
            csvMediaId, callbackUrl, method)
    def get_result(self, jobId):
        return self.determine_wrapper(None, qyUsers.get_result,
            jobId)



from requests import get

from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry
from ..base.common import (
    TokenClass as _TokenClass, ServerListClass as _ServerListClass)
from ..requests import requests

class TokenClass(_TokenClass):
    def __init__(self, core):
        _TokenClass.__init__(self, core, SERVER_URL + 
            '/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s',
            lambda x: x.config.appId)

class ServerListClass(_ServerListClass):
    def __init__(self, core, tokenClass):
        _ServerListClass.__init__(self, core, tokenClass, 
            '{}/cgi-bin/getcallbackip?access_token=%s'.format(SERVER_URL))

def clear_quota(accessToken=None):
    data = {'appid': server.config.appId}
    r = requests.post('%s/cgi-bin/clear_quota?access_token=%s' %
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    return r



import logging, json

from ..requests import requests
from itchatmp.utils import retry, encode_send_dict
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, MUSIC, TEXT, NEWS, CARD)
from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

def get(accessToken=None):
    r = requests.post('%s/cgi-bin/customservice/getkflist?access_token=%s'
        % (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'kf_list' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def add(accountDict, accessToken=None):
    data = encode_send_dict(accountDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/customservice/kfaccount/add?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def update(accountDict, accessToken=None):
    data = encode_send_dict(accountDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/customservice/kfaccount/add?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def delete(accountDict, accessToken=None):
    data = encode_send_dict(accountDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/customservice/kfaccount/del?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def set_head_image(openedFile, kfAccount, accessToken=None):
    r = requests.post('%s/customservice/kfaccount/uploadheadimg?'
        'access_token=%s&kf_account=%s' % 
        (SERVER_URL, accessToken, kfAccount),
        files={'file': openedFile})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def send(msgType, mediaId, additionalDict={}, toUserId='', accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict:
        return msgDict
    msgDict['touser'] = toUserId
    data = encode_send_dict(msgDict)
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/message/custom/send?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def __form_send_dict(msgType, mediaId, additionalDict):
    ''' additionalDict will be formatted to give full compacity to input
    thumb_media_id, ThumbMediaId, thumbmediaid are all supported '''
    if not msgType in (IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC):
        return ReturnValue({'errcode': 40008,})
    if additionalDict: # format additionalDict
        for key in ('text', 'image', 'voice', 'video',
                'music', 'news', 'mpnews', 'wxcard'):
            if key in additionalDict and isinstance(additionalDict[key], dict):
                for k, v in additionalDict[key].items():
                    if k not in additionalDict:
                        additionalDict[k] = v
        additionalDict = {k.lower().replace('_', ''): v
            for k, v in additionalDict.items()}
        if 'cardid' in additionalDict:
            additionalDict['mediaid'] = additionalDict['cardid']
        if 'introduction' in additionalDict:
            additionalDict['description'] = additionalDict['introduction']
    if msgType == VIDEO:
        if not all((k in additionalDict for k in ('title', 'description'))):
            return ReturnValue({'errcode': -10003, 'errmsg': 
                'additionalDict for type VIDEO should be: ' + 
                "{'ThumbMediaId': 'id', 'Title': 'title', " +
                "'Description': 'des'}"})
    elif msgType == MUSIC:
        if not all((k in additionalDict for k in ('title', 'description',
                'musicurl', 'hqmusicurl', 'thumbmediaid'))):
            return ReturnValue({'errcode': -10003, 'errmsg': 
                'additionalDict for type MUSIC should be: ' + 
                "{'MusicUrl' : 'url', 'HqMusicUrl' : 'url', " +
                "'Title': 'title', 'Description': 'des', " +
                "'ThumbMediaId': 'id'}"})
    elif msgType == NEWS:
        if 'articles' not in additionalDict:
            msgType = 'mpnews'
    return {
        TEXT: {'text': {'content': mediaId}, 'msgtype': 'text'},
        IMAGE: {'image': {'media_id': mediaId}, 'msgtype': 'image'},
        VOICE: {'voice': {'media_id': mediaId}, 'msgtype': 'voice'},
        VIDEO: {'video': {'media_id': mediaId,
                'thumb_media_id': additionalDict.get('thumbmediaid', ''),
                'title': additionalDict.get('title', ''),
                'description': additionalDict.get('description', '')},
            'msgtype': 'video'},
        MUSIC: {'music': {
                'title': additionalDict.get('title', ''),
                'description': additionalDict.get('description', ''),
                'musicurl': additionalDict.get('musicurl', ''),
                'hqmusicurl': additionalDict.get('hqmusicurl', ''),
                'thumb_media_id': additionalDict.get('thumbmediaid', ''), },
            'msgtype': 'music'},
        NEWS: {'news':{'articles': additionalDict.get('articles', [])},
            'msgtype': 'news'},
        'mpnews': {'mpnews':{'media_id': mediaId}, 'msgtype': 'mpnews'},
        CARD: {'wxcard': {'card_id': mediaId}, 'msgtype': 'wxcard'},
        }[msgType]



from .common import BaseController
from .mpapi.mp import customerservice as mpCs

class CustomerService(BaseController):
    def get(self):
        return self.determine_wrapper(mpCs.get, None)
    def add(self, accountDict, autoDecide=True):
        return self.determine_wrapper(mpCs.add, None,
            accountDict, autoDecide)
    def update(self, accountDict, autoDecide=True):
        return self.determine_wrapper(mpCs.update, None,
            accountDict, autoDecide)
    def delete(self, accountDict, autoDecide=True):
        return self.determine_wrapper(mpCs.delete, None,
            accountDict, autoDecide)
    def set_head_image(self, openedFile, kfAccount):
        return self.determine_wrapper(mpCs.set_head_image, None,
            openedFile, kfAccount)
    def send(self, msgType, mediaId, additionalDict={}, toUserId=''):
        return self.determine_wrapper(mpCs.send, None,
            msgType, mediaId, additionalDict, toUserId)



from .common import BaseController
from .mpapi.mp import wrapped as mpWrapped

class Wrapped(BaseController):
    def send(self, msg, toUserName, mediaId=None):
        return self.determine_wrapper(mpWrapped.send, None,
            self.core, msg, toUserName, mediaId)



#coding=utf8
import sys, collections

from tornado.gen import Future

TRANSLATE = 'Chinese'
TRANSLATION = {}

class NotDict(collections.MutableMapping):
    def __init__(self, *args, **kwargs):
        self.store = dict()
        self.update(dict(*args, **kwargs))
    def __getitem__(self, key):
        return self.store[self.__keytransform__(key)]
    def __setitem__(self, key, value):
        self.store[self.__keytransform__(key)] = value
    def __delitem__(self, key):
        del self.store[self.__keytransform__(key)]
    def __iter__(self):
        return iter(self.store)
    def __len__(self):
        return len(self.store)
    def __keytransform__(self, key):
        return key

class ReturnValue(NotDict, Future):
    def __init__(self, returnValueDict={}):
        # make returnvalue yieldable
        NotDict.__init__(self, **returnValueDict)
        Future.__init__(self)
        self.set_result(self)
        # init
        self._wrap_result = None
        if TRANSLATE:
            self['rawmsg'] = self.get('errmsg', '')
            self['errmsg'] = \
                TRANSLATION[TRANSLATE].get(self.get('errcode', '')) \
                or self.get('errmsg', u'没有errmsg')
            if not self['rawmsg']: self['rawmsg'] = self['errmsg']
    def __nonzero__(self):
        return self.get('errcode', -1) == 0
    def __bool__(self):
        return self.__nonzero__()
    def __str__(self):
        return '{%s}' % ', '.join(
            ['%s: %s' % (repr(k),repr(v)) for k,v in self.items()])
    def __repr__(self):
        return '<ItchatmpReturnValue: %s>' % self.__str__()

TRANSLATION = {
    'Chinese': {
        -10001: u'参数中所有非ascii字符串需要用Unicode形式传递，如：u"中文"',
        -10002: u'文件读取失败，请以"rb"打开文件并作为参数传入',
        -10003: u'传入参数有误',
        -10004: u'文件无法读取，请检查文件位置与权限',
        -10005: u'返回值不合法',
        -1: u'系统繁忙，此时请开发者稍候再试',
        0: u'请求成功',
        40001: u'获取access_token时AppSecret错误，或者access_token无效。请开发者认真比对AppSecret的正确性，或查看是否正在为恰当的公众号调用接口',
        40002: u'不合法的凭证类型',
        40003: u'不合法的OpenID，请开发者确认OpenID（该用户）是否已关注公众号，或是否是其他公众号的OpenID',
        40004: u'不合法的媒体文件类型',
        40005: u'不合法的文件类型',
        40006: u'不合法的文件大小',
        40007: u'不合法的媒体文件id',
        40008: u'不合法的消息类型',
        40009: u'不合法的图片文件大小',
        40010: u'不合法的语音文件大小',
        40011: u'不合法的视频文件大小',
        40012: u'不合法的缩略图文件大小',
        40013: u'不合法的AppID，请开发者检查AppID的正确性，避免异常字符，注意大小写',
        40014: u'不合法的access_token，请开发者认真比对access_token的有效性（如是否过期），或查看是否正在为恰当的公众号调用接口',
        40015: u'不合法的菜单类型',
        40016: u'不合法的按钮个数',
        40017: u'不合法的按钮个数',
        40018: u'不合法的按钮名字长度',
        40019: u'不合法的按钮KEY长度',
        40020: u'不合法的按钮URL长度',
        40021: u'不合法的菜单版本号',
        40022: u'不合法的子菜单级数',
        40023: u'不合法的子菜单按钮个数',
        40024: u'不合法的子菜单按钮类型',
        40025: u'不合法的子菜单按钮名字长度',
        40026: u'不合法的子菜单按钮KEY长度',
        40027: u'不合法的子菜单按钮URL长度',
        40028: u'不合法的自定义菜单使用用户',
        40029: u'不合法的oauth_code',
        40030: u'不合法的refresh_token',
        40031: u'不合法的openid列表',
        40032: u'不合法的openid列表长度',
        40033: u'不合法的请求字符，不能包含\\uxxxx格式的字符',
        40035: u'不合法的参数',
        40038: u'不合法的请求格式',
        40039: u'不合法的URL长度',
        40050: u'不合法的分组id',
        40051: u'分组名字不合法',
        40117: u'分组名字不合法',
        40118: u'media_id大小不合法',
        40119: u'button类型错误',
        40120: u'button类型错误',
        40121: u'不合法的media_id类型',
        40132: u'微信号不合法',
        40137: u'不支持的图片格式',
        40155: u'请勿添加其他公众号的主页链接',
        41001: u'缺少access_token参数',
        41002: u'缺少appid参数',
        41003: u'缺少refresh_token参数',
        41004: u'缺少secret参数',
        41005: u'缺少多媒体文件数据',
        41006: u'缺少media_id参数',
        41007: u'缺少子菜单数据',
        41008: u'缺少oauth code',
        41009: u'缺少openid',
        42001: u'access_token超时，请检查access_token的有效期，请参考基础支持-获取access_token中，对access_token的详细机制说明',
        42002: u'refresh_token超时',
        42003: u'oauth_code超时',
        42007: u'用户修改微信密码，accesstoken和refreshtoken失效，需要重新授权',
        43001: u'需要GET请求',
        43002: u'需要POST请求',
        43003: u'需要HTTPS请求',
        43004: u'需要接收者关注',
        43005: u'需要好友关系',
        43019: u'需要将接收者从黑名单中移除',
        44001: u'多媒体文件为空',
        44002: u'POST的数据包为空',
        44003: u'图文消息内容为空',
        44004: u'文本消息内容为空',
        45001: u'多媒体文件大小超过限制',
        45002: u'消息内容超过限制',
        45003: u'标题字段超过限制',
        45004: u'描述字段超过限制',
        45005: u'链接字段超过限制',
        45006: u'图片链接字段超过限制',
        45007: u'语音播放时间超过限制',
        45008: u'图文消息超过限制',
        45009: u'接口调用超过限制',
        45010: u'创建菜单个数超过限制',
        45011: u'API调用太频繁，请稍候再试',
        45015: u'回复时间超过限制',
        45016: u'系统分组，不允许修改',
        45017: u'分组名字过长',
        45018: u'分组数量超过上限',
        45047: u'客服接口下行条数超过上限',
        45056: u'创建的标签数过多，请注意不能超过100个',
        45057: u'该标签下粉丝数超过10w，不允许直接删除',
        45058: u'不能修改0/1/2这三个系统默认保留的标签',
        45059: u'有粉丝身上的标签数已经超过限制', 
        45157: u'标签名非法，请注意不能和其他标签重名',
        45158: u'标签名长度超过30个字节',
        45159: u'非法的tag_id',
        46001: u'不存在媒体数据',
        46002: u'不存在的菜单版本',
        46003: u'不存在的菜单数据',
        46004: u'不存在的用户',
        47001: u'解析JSON/XML内容错误',
        48001: u'api功能未授权，请确认公众号已获得该接口，可以在公众平台官网-开发者中心页中查看接口权限',
        48002: u'粉丝拒收消息（粉丝在公众号选项中，关闭了“接收消息”）',
        48004: u'api接口被封禁，请登录mp.weixin.qq.com查看详情',
        48005: u'api禁止删除被自动回复和自定义菜单引用的素材',
        48006: u'api禁止清零调用次数，因为清零次数达到上限',
        49003: u'传入的openid不属于此AppID',
        50001: u'用户未授权该api',
        50002: u'用户受限，可能是违规后接口被封禁',
        61451: u'参数错误(invalid parameter)',
        61452: u'无效客服账号(invalid kf_account)',
        61453: u'客服帐号已存在(kf_account exsited)',
        61454: u'客服帐号名长度超过限制(仅允许10个英文字符，不包括@及@后的公众号的微信号)(invalid   kf_acount length)',
        61455: u'客服帐号名包含非法字符(仅允许英文+数字)(illegal character in     kf_account)',
        61456: u'客服帐号个数超过限制(10个客服账号)(kf_account count exceeded)',
        61457: u'无效头像文件类型(invalid   file type)',
        61450: u'系统错误(system error)',
        61500: u'日期格式错误',
        65301: u'不存在此menuid对应的个性化菜单',
        65302: u'没有相应的用户',
        65303: u'没有默认菜单，不能创建个性化菜单',
        65304: u'MatchRule信息为空',
        65305: u'个性化菜单数量受限',
        65306: u'不支持个性化菜单的帐号',
        65307: u'个性化菜单信息为空',
        65308: u'包含没有响应类型的button',
        65309: u'个性化菜单开关处于关闭状态',
        65310: u'填写了省份或城市信息，国家信息不能为空',
        65311: u'填写了城市信息，省份信息不能为空',
        65312: u'不合法的国家信息',
        65313: u'不合法的省份信息',
        65314: u'不合法的城市信息',
        65316: u'该公众号的菜单设置了过多的域名外跳（最多跳转到3个域名的链接）',
        65317: u'不合法的URL',
        9001001: u'POST数据参数不合法',
        9001002: u'远端服务不可用',
        9001003: u'Ticket不合法',
        9001004: u'获取摇周边用户信息失败',
        9001005: u'获取商户信息失败',
        9001006: u'获取OpenID失败',
        9001007: u'上传文件缺失',
        9001008: u'上传素材的文件类型不合法',
        9001009: u'上传素材的文件尺寸不合法',
        9001010: u'上传失败',
        9001020: u'帐号不合法',
        9001021: u'已有设备激活率低于50%，不能新增设备',
        9001022: u'设备申请数不合法，必须为大于0的数字',
        9001023: u'已存在审核中的设备ID申请',
        9001024: u'一次查询设备ID数量不能超过50',
        9001025: u'设备ID不合法',
        9001026: u'页面ID不合法',
        9001027: u'页面参数不合法',
        9001028: u'一次删除页面ID数量不能超过10',
        9001029: u'页面已应用在设备中，请先解除应用关系再删除',
        9001030: u'一次查询页面ID数量不能超过50',
        9001031: u'时间区间不合法',
        9001032: u'保存设备与页面的绑定关系参数错误',
        9001033: u'门店ID不合法',
        9001034: u'设备备注信息过长',
        9001035: u'设备申请参数不合法',
        9001036: u'查询起始值begin不合法',
    },
}



from .common import BaseController
from .mpapi.mp import messages as mpMsg
from .mpapi.qy import messages as qyMsg

class Messages(BaseController):
    def send_some(self, msgType, mediaId, additionalDict={},
            targetIdList=[], partyIdList=[], tagIdList=[],
            agentId=None):
        return self.determine_wrapper(mpMsg.send_some, qyMsg.send_some,
            msgType, mediaId, additionalDict,
            targetIdList, partyIdList, tagIdList, agentId)
    def send_all(self, msgType, mediaId, additionalDict={},
            tagId=None, agentId=None):
        return self.determine_wrapper(mpMsg.send_all, qyMsg.send_all,
            msgType, mediaId, additionalDict, tagId, agentId)
    def preview(self, msgType, mediaId, additionalDict={}, toUserId=None, toWxAccount=None):
        return self.determine_wrapper(mpMsg.preview, None,
            msgType, mediaId, additionalDict, toUserId, toWxAccount)
    def delete(self, msgId):
        return self.determine_wrapper(mpMsg.delete, None,
            msgId)
    def get(self, msgId):
        return self.determine_wrapper(mpMsg.get, None,
            msgId)
    def upload(self, fileType, fileDir, additionalDict={}, permanent=False):
        return self.determine_wrapper(mpMsg.upload, qyMsg.upload,
            fileType, fileDir, additionalDict, permanent)
    def download(self, mediaId):
        return self.determine_wrapper(mpMsg.download, qyMsg.download,
            mediaId)
    def get_material(self, mediaId):
        return self.determine_wrapper(mpMsg.get_material, qyMsg.get_material,
            mediaId)
    def delete_material(self, mediaId):
        return self.determine_wrapper(mpMsg.delete_material, qyMsg.delete_material,
            mediaId)
    def get_material_count(self):
        return self.determine_wrapper(mpMsg.get_material_count, qyMsg.get_material_count)
    def batchget_material(self, fileType, offset=0, count=20):
        return self.determine_wrapper(mpMsg.batchget_material, qyMsg.batchget_material,
            fileType, offset, count)
    def create_news(self, newsDict, permanent=False):
        return self.determine_wrapper(mpMsg.create_news, qyMsg.create_news,
            newsDict, permanent)
    def update_news(self, mediaId, newsDict, index=0):
        return self.determine_wrapper(mpMsg.update_news, qyMsg.update_news,
            mediaId, newsDict, index)
    def get_image_url(self, openedFile):
        return self.determine_wrapper(mpMsg.get_image_url, qyMsg.get_image_url,
            openedFile)
    def get_autoreply(self):
        return self.determine_wrapper(mpMsg.get_autoreply, None)



import copy
import logging
import traceback

import tornado
from tornado import gen
from tornado.web import RequestHandler
from concurrent.futures import ThreadPoolExecutor

from itchatmp.config import SERVER_WAIT_TIME, COROUTINE
from itchatmp.content import (NORMAL, COMPATIBLE, SAFE,
                              TEXT, INCOME_MSG, OUTCOME_MSG)
from itchatmp.views import (
    deconstruct_msg, construct_msg, reply_msg_format,
    decrypt_msg, encrypt_msg, oauth)
from itchatmp.controllers.envtest import env_test
from itchatmp.exceptions import ParameterError
from itchatmp.log import set_logging

logger = logging.getLogger('itchatmp')


def load_register(core):
    core.update_config = update_config
    core.run = run
    core.msg_register = msg_register


def construct_get_post_fn(core):
    def get_fn(handler):
        ''' only for verifying server
            return echostr if verify
            return greeting words if not
        '''
        if core.filterRequest and not core.filter_request(handler.request):
            logger.debug('A request from unknown ip is filtered')
            return 'Greeting from itchatmp!'
        else:
            return verify_echostr(core, handler) or 'Greeting from itchatmp!'

    def sync_post_fn(handler):
        if core.filterRequest and not core.filter_request(handler.request):
            logger.debug('A request from unknown ip is filtered')
            return None, None
        else:
            msgDict = deconstruct_msg(handler.request.body)
            isActualEncrypt = 'Encrypt' in msgDict
            tns = get_tns(core, handler)
            msgDict = verify_message(core, handler, tns, msgDict)
            if not msgDict:
                logger.debug('Ignore a request because verify failed')
            else:
                reply_fn = get_reply_fn(core, msgDict['MsgType'])
                if reply_fn is None:
                    return None, None
                try:
                    reply = reply_fn(copy.deepcopy(msgDict))
                except Exception as e:
                    logger.warning(traceback.format_exc())
                else:  # if nothing goes wrong
                    if reply:
                        return verify_reply(core, tns, reply, msgDict, isActualEncrypt)
        return None, None

    @gen.coroutine
    def coroutine_post_fn(handler):
        if core.filterRequest and not core.filter_request(handler.request):
            logger.debug('A request from unknown ip is filtered')
        else:
            msgDict = deconstruct_msg(handler.request.body)
            tns = get_tns(core, handler)
            isActualEncrypt = 'Encrypt' in msgDict
            msgDict = verify_message(core, handler, tns, msgDict)
            if not msgDict:
                logger.debug('Ignore a request because verify failed')
            else:
                reply_fn = get_reply_fn(core, msgDict['MsgType'])
                if reply_fn is None:
                    raise gen.Return((None, None))
                try:
                    reply = yield reply_fn(copy.deepcopy(msgDict))
                except Exception as e:
                    logger.warning(traceback.format_exc())
                else:  # if nothing goes wrong
                    if reply:
                        r = yield verify_reply(core, tns, reply, msgDict, isActualEncrypt)
                        raise gen.Return(r)
        raise gen.Return((None, None))
    return get_fn, coroutine_post_fn if COROUTINE else sync_post_fn


def get_tns(core, handler):
    if handler.get_argument('msg_signature', ''):
        tns = [handler.get_argument(key, '') for
               key in ('timestamp', 'nonce', 'msg_signature')]
    else:
        tns = [handler.get_argument(key, '') for
               key in ('timestamp', 'nonce', 'signature')]
    return tns


def verify_echostr(core, handler):
    '''
        verify signature and return echostr if valid
        if not, None will be returned
    '''
    tns = get_tns(core, handler)
    echostr = handler.get_argument('echostr', '')
    if handler.get_argument('msg_signature', ''):
        if oauth(*(tns + [echostr, core.config.token])):
            msgDict = decrypt_msg(*(tns + [core.config, {'echostr': echostr}]))
            echostr = msgDict.get('echostr')
    else:
        valid = oauth(*(tns + [core.config.token]))
        if not valid:
            echostr = None
    return echostr


def verify_message(core, handler, tns, msgDict):
    '''
        verify signature and return decrypted message if valid
        if not, None will be returned
    '''
    if handler.get_argument('msg_signature', ''):
        valid = oauth(*(tns
                        + [core.config.token, msgDict.get('Encrypt', '')]))
    else:
        valid = oauth(*(tns + [core.config.token]))
    if valid:
        if core.config.encryptMode == SAFE:
            msgDict = decrypt_msg(*(tns + [core.config, msgDict]))
    else:
        msgDict = {}
    return msgDict


if COROUTINE:
    @gen.coroutine
    def verify_reply(core, tns, reply, msgDict, isActualEncrypt):
        reply = reply_msg_format(reply)
        if reply:
            if reply.get('MsgType') in OUTCOME_MSG:
                reply['ToUserName'] = msgDict['FromUserName']
                reply['FromUserName'] = msgDict['ToUserName']
                if 'FileDir' in reply and reply['MsgType'] != TEXT:
                    r = yield core.upload(reply['MsgType'], reply['FileDir'])
                    if not r:
                        logger.warning(r)
                        raise gen.Return((None, None))
                    else:
                        reply['MediaId'] = r['media_id']
                if core.config.encryptMode == SAFE and isActualEncrypt:
                    raise gen.Return((encrypt_msg(*(tns
                                                    + [core.config, reply])), reply))
                else:
                    raise gen.Return((construct_msg(reply), reply))
            else:
                logger.warning('Reply is invalid: unknown MsgType')
        else:
            logger.warning('Reply is invalid: %s' % reply.get('errmsg'))
        raise gen.Return((None, None))
else:
    def verify_reply(core, tns, reply, msgDict, isActualEncrypt):
        reply = reply_msg_format(reply)
        if reply:
            if reply.get('MsgType') in OUTCOME_MSG:
                reply['ToUserName'] = msgDict['FromUserName']
                reply['FromUserName'] = msgDict['ToUserName']
                if 'FileDir' in reply and reply['MsgType'] != TEXT:
                    r = core.upload(reply['MsgType'], reply['FileDir'])
                    if not r:
                        logger.warning(r)
                        return None, None
                    else:
                        reply['MediaId'] = r['media_id']
                if core.config.encryptMode == SAFE and isActualEncrypt:
                    return encrypt_msg(*(tns
                                         + [core.config, reply])), reply
                else:
                    return construct_msg(reply), reply
            else:
                logger.warning('Reply is invalid: unknown MsgType')
        else:
            logger.warning('Reply is invalid: %s' % reply.get('errmsg'))
        return None, None


def construct_handler(core, isWsgi):
    get_fn, post_fn = construct_get_post_fn(core)

    class BaseHandler(RequestHandler):
        def initialize(self):
            self.closed = False

        def on_connection_close(self):
            self.closed = True

        def get(self):
            self.finish(get_fn(self))
    if isWsgi:
        class MainHandler(BaseHandler):
            def post(self):
                r, rawReply = post_fn(self)
                if self.closed:  # server has stopped waiting
                    if rawReply:
                        r = core.send(rawReply, rawReply.get('ToUserName', ''))
                        if not r:
                            logger.warning('Reply error: %s' % r.get('errmsg', ''))
                else:
                    self.finish(r)
    else:
        ioLoop = core.ioLoop
        if COROUTINE:
            class MainHandler(BaseHandler):
                @tornado.gen.coroutine
                def post(self):
                    def time_out_callback():
                        self.finish()
                        self.closed = True
                    timeoutHandler = ioLoop.call_later(SERVER_WAIT_TIME,
                                                       time_out_callback)
                    r, rawReply = yield post_fn(self)
                    ioLoop.remove_timeout(timeoutHandler)
                    if self.closed:
                        if rawReply:
                            r = yield core.send(rawReply, rawReply.get('ToUserName', ''))
                            if not r:
                                logger.warning('Reply error: %s' % r.get('errmsg', ''))
                    else:
                        self.finish(r)
        else:
            threadPool = ThreadPoolExecutor(core.threadPoolNumber)

            class MainHandler(BaseHandler):
                @tornado.gen.coroutine
                def post(self):
                    def time_out_callback():
                        self.finish()
                        self.closed = True
                    timeoutHandler = ioLoop.call_later(SERVER_WAIT_TIME,
                                                       time_out_callback)
                    r, rawReply = yield threadPool.submit(post_fn, self)
                    ioLoop.remove_timeout(timeoutHandler)
                    if self.closed:
                        if rawReply:
                            r = yield threadPool.submit(core.send,
                                                        (rawReply, rawReply.get('ToUserName', '')))
                            if not r:
                                logger.warning('Reply error: %s' % r.get('errmsg', ''))
                    else:
                        self.finish(r)
    return MainHandler


def update_config(self, config=None, atStorage=None, userStorage=None,
                  filterRequest=None, threadPoolNumber=None):
    self.config = config or self.config
    self.atStorage = atStorage or self.atStorage
    self.userStorage = userStorage or self.userStorage
    self.filterRequest = filterRequest or self.filterRequest
    self.threadPoolNumber = threadPoolNumber or self.threadPoolNumber


def run(self, isWsgi=False, debug=True, port=80):
    self.isWsgi = isWsgi
    self.debug = debug
    if debug:
        set_logging(loggingLevel=logging.DEBUG)
    MainHandler = construct_handler(self, isWsgi)
    app = tornado.web.Application(
        [('/', MainHandler)], debug=debug)
    logger.info('itchatmp started!%s' % (
        ' press Ctrl+C to exit.' if debug else ''))
    port = int(port)
    env_test(port)
    app.listen(port)
    try:
        self.ioLoop.start()
    except:
        logger.info('Bye~')
        self.ioLoop.stop()


def msg_register(self, msgType):
    ''' decorator to register message handlers
     * msgType can be type like TEXT or a list of them
     * register twice will override the older one
    '''
    def _msg_register(fn):
        if COROUTINE:
            fn = gen.coroutine(fn)
        msgTypeList = msgType if isinstance(msgType, list) else [msgType]
        for t in msgTypeList:
            if t in INCOME_MSG:
                self._replyFnDict[t] = fn
            else:
                raise ParameterError(
                    'Known type register "%s"' % t)
        return fn
    return _msg_register


def get_reply_fn(core, msgType):
    return core._replyFnDict.get(msgType)



import os, logging, traceback
import hashlib, struct
from base64 import b64decode, b64encode

try:
    from Crypto.Cipher import AES
except ImportError:
    from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
    from cryptography.hazmat.backends import default_backend
    def aes_encode(key, data):
        cryptor = Cipher(algorithms.AES(key), modes.CBC(key[:16]),
            backend=default_backend()).encryptor()
        return b64encode(cryptor.update(data) + cryptor.finalize())
    def aes_decode(key, data):
        cryptor = Cipher(algorithms.AES(key), modes.CBC(key[:16]),
            backend=default_backend()).decryptor()
        return cryptor.update(b64decode(data)) + cryptor.finalize()
else:
    def aes_encode(key, data):
        cryptor = AES.new(key, AES.MODE_CBC, key[:16])
        return b64encode(cryptor.encrypt(data))
    def aes_decode(key, data):
        cryptor = AES.new(key, AES.MODE_CBC, key[:16])
        return cryptor.decrypt(b64decode(data))

from itchatmp.content import ENCRYPT
from itchatmp.views import deconstruct_msg, construct_msg

logger = logging.getLogger('itchatmp')

def decrypt_msg(timestamp, nonce, signature, config, msgDict):
    ''' decrypt msg from wechat, use AES_CBC decryption
        return a dict contains encrypted information
        if decrypt failed, will return an empty dict
        pass {'echostr': ECHOSTR} into msgDict to decrypt Cop mp oauth
    '''
    if 'echostr' in msgDict:
        msgDict['Encrypt'] = msgDict['echostr']
    elif msgDict.get('MsgType') != ENCRYPT:
        return msgDict
    try:
        text = aes_decode(config._encodingAesKey, msgDict['Encrypt'])
        text = text[16:-(text[-1] if isinstance(text[-1], int) else ord(text[-1]))]
        xmlLen = struct.unpack('>I', text[:4])[0]
        xmlContent = text[4:xmlLen + 4].decode('utf8')
        fromAppid = text[xmlLen + 4:].decode('utf8')
    except:
        logger.debug(traceback.format_exc())
        return {}
    # Check appId
    if fromAppid not in (config.appId, config.copId):
        logger.debug('A message from wrong appid is filtered when decrypt: %s' % fromAppid)
        return {}
    if 'echostr' in msgDict:
        return {'echostr': xmlContent}
    else:
        return deconstruct_msg(xmlContent)

def encrypt_msg(timestamp, nonce, signature, config, replyDict):
    ''' encrypt msg for sending to wechat
     * use AES_CBC encryption
     * return a string ready for sending
     * as in construct_msg, string in replyDict should be unicode
    '''
    text = construct_msg(replyDict).encode('utf8')
    text = os.urandom(16) + struct.pack('>I', len(text)) +\
        text + config.appId.encode('utf8')
    paddingAmount = 32 - (len(text) % 32)
    text += chr(paddingAmount).encode('utf8') * paddingAmount
    text = aes_encode(config._encodingAesKey, text)
    # Encrypt generated
    s = [i.encode('utf8') for i in (timestamp, nonce, config.token)]
    s += [text]; s.sort(); s = b''.join(s)
    # Signature generated
    return construct_msg({
            'FromUserName': replyDict['FromUserName'],
            'ToUserName': replyDict['ToUserName'],
            'MsgType': ENCRYPT,
            'Encrypt': text.decode('utf8'),
            'MsgSignature': hashlib.sha1(s).hexdigest(),
            'TimeStamp': timestamp,
            'Nonce': nonce,
        }, )

def oauth(timestamp, nonce, signature, token, echostr=None):
    ''' determine whether signature of request is right
     * both get and post functions need oauth
     * msgs we generate for sending don't match this fn
     * for Cop mp, we need echostr as well
    '''
    s = [timestamp, nonce, token]
    if echostr is not None: s.append(echostr)
    s.sort(); s = ''.join(s).encode('utf8')
    return hashlib.sha1(s).hexdigest() == signature



import logging

from ..requests import requests
from .common import access_token
from itchatmp.config import COMPANY_URL
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, MUSIC, TEXT, NEWS, CARD)
from itchatmp.utils import retry, encode_send_dict
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def authorize_user(userId, accessToken=None):
    params = {
        'access_token': accessToken,
        'userid': userId, }
    r = requests.get('%s/cgi-bin/user/authsucc' % COMPANY_URL, params=params).json()
    return ReturnValue(r)

@access_token
def create_department(name, parentId=1, order=None, id=None, accessToken=None):
    data = {
        'name': name,
        'parentid': parentId, }
    if order is not None: data['order'] = order
    if id is not None: data['id'] = id
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/department/create?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def update_department(id, name=None, parentId=None, order=None, accessToken=None):
    data = {'id': id}
    if name is not None: data['name'] = name
    if parentId is not None: data['parentid'] = parentId
    if order is not None: data['order'] = order
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/department/update?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def delete_department(id, accessToken=None):
    params = {
        'access_token': accessToken,
        'id': id, }
    r = requests.get('%s/cgi-bin/department/delete' % COMPANY_URL, params=params).json()
    return ReturnValue(r)

@access_token
def get_departments(parentId, accessToken=None):
    params = {
        'access_token': accessToken,
        'id': parentId, }
    r = requests.get('%s/cgi-bin/department/list' % COMPANY_URL, params=params).json()
    return ReturnValue(r)

@access_token
def create_user(userId, name, departmentIdList,
        position=None, mobile=None, gender=None, email=None,
        weixinId=None, headImgId=None, extAttr=None,
        accessToken=None):
    data = {
        'userid': userId               , 'name': name         ,
        'department': departmentIdList , 'position': position ,
        'mobile': mobile               , 'gender': gender     ,
        'email': email                 , 'weixinid': weixinId ,
        'avatar_mediaid': headImgId    , 'extattr': extAttr   , }
    for k in list(data):
        if data[k] is None: del data[k]
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/user/create?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def update_user(userId, name=None, departmentIdList=None,
        position=None, mobile=None, gender=None, email=None,
        weixinId=None, headImgId=None, extAttr=None,
        accessToken=None):
    data = {
        'userid': userId               , 'name': name         ,
        'department': departmentIdList , 'position': position ,
        'mobile': mobile               , 'gender': gender     ,
        'email': email                 , 'weixinid': weixinId ,
        'avatar_mediaid': headImgId    , 'extattr': extAttr   , }
    for k in list(data):
        if data[k] is None: del data[k]
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/user/update?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def delete_users(userId, accessToken=None):
    ''' delete user using userId
     * userId can be a list or only one userId
    '''
    if isinstance(userId, list):
        data = {'useridlist': userId}
        data = encode_send_dict(data)
        if data is None: return ReturnValue({'errcode': -10001})
        url = '%s/cgi-bin/department/batchdelete?access_token=%s' % (COMPANY_URL, accessToken)
        r = requests.post(url, data=data).json()
    else:
        params = {
            'access_token': accessToken,
            'userid': userId, }
        url = '%s/cgi-bin/department/delete'
        r = requests.get(url % COMPANY_URL, params=params).json()
    return ReturnValue(r)

def get_user_info(userId):
    ''' get info of a user or a list of users
     * userId can be a list or only one userId
     * this is for creating api similiar to massive platform
    '''
    @access_token
    def _get_user_info(userId):
        params = {
            'access_token': accessToken,
            'userid': userId, }
        r = requests.get('%s/cgi-bin/user/get' % COMPANY_URL, params=params).json()
        return ReturnValue(r)
    if isinstance(userId, list):
        userDict = {'user_info_list': []}
        for id in userId:
            r = _get_user_info(id)
            if r:
                userDict['user_info_list'].append(r)
            else:
                userDict['errcode'] = r['errcode']
                userDict['errmsg'] = r['errmsg']
                break
        return ReturnValue(userDict)
    else:
        return _get_user_info(userId)

@access_token
def get_users(nextOpenId='', departmentId=None, fetchChild=False, status=4, accessToken=None):
    ''' get users of the department
     * nextOpenId is for mp api
    '''
    if departmentId is None:
        return ReturnValue({'errcode': 40035, 'errmsg': 'departmentId must be set',})
    params = {
        'access_token'  : accessToken,
        'department_id' : departmentId,
        'fetch_child'   : int(fetchChild),
        'status'        : status, }
    r = requests.get('%s/cgi-bin/user/simplelist' % SERVER_URL, params=params).json()
    return ReturnValue(r)

@access_token
def get_detailed_users(nextOpenId='',
        departmentId=None, fetchChild=False, status=4, accessToken=None):
    ''' get users of the department
     * nextOpenId is for mp api
    '''
    if departmentId is None:
        return ReturnValue({'errcode': 40035, 'errmsg': 'departmentId must be set',})
    params = {
        'access_token'  : accessToken,
        'department_id' : departmentId,
        'fetch_child'   : int(fetchChild),
        'status'        : status, }
    r = requests.get('%s/cgi-bin/user/list' % SERVER_URL, params=params).json()
    return ReturnValue(r)

@access_token
def create_tag(name, id=None, accessToken=None):
    data = {'tagname': name}
    if id is not None: data['tagid'] = id
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tag/create?access_token=%s'
        % (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def get_tags(accessToken=None):
    r = requests.get('%s/cgi-bin/tag/list?access_token=%s'
        % (SERVER_URL, accessToken)).json()
    return ReturnValue(r)

@access_token
def update_tag(id, name, accessToken=None):
    data = encode_send_dict({'tagid': id, 'tagname': name, })
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tag/update?access_token=%s'
        % (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def delete_tag(id, accessToken=None):
    params = {
        'access_token': accessToken,
        'tagid': id, }
    r = requests.get('%s/cgi-bin/tag/delete' % COMPANY_URL, params=params).json()
    return ReturnValue(r)

@access_token
def get_users_of_tag(id, nextOpenId='', accessToken=None):
    params = {
        'access_token': accessToken,
        'tagid': id, }
    r = requests.get('%s/cgi-bin/tag/get' % COMPANY_URL, params=params).json()
    return ReturnValue(r)

@access_token
def add_users_into_tag(id, userIdList=None, partyList=None, accessToken=None):
    if not (userIdList or partyList):
        return ReturnValue({'errcode': 40035,
            'errmsg': 'either userId or partyList should be set'})
    data = {'tagid': id}
    if userIdList:
        data['userlist'] = userIdList
    if partyList:
        data['partylist'] = partyList
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tag/addtagusers?access_token=%s'
        % (COMPANY_URL, accessToken), data=data).json()
    if 'invalidlist' in r or 'invalidparty' in r: r['errcode'] = 40070
    return ReturnValue(r)

@access_token
def delete_users_of_tag(id, userIdList=None, partyList=None, accessToken=None):
    if not (userIdList or partyList):
        return ReturnValue({'errcode': 40035,
            'errmsg': 'either userId or partyList should be set'})
    data = {'tagid': id}
    if userIdList:
        data['userlist'] = userIdList
    if partyList:
        data['partylist'] = partyList
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tag/deltagusers?access_token=%s'
        % (COMPANY_URL, accessToken), data=data).json()
    if 'invalidlist' in r or 'invalidparty' in r: r['errcode'] = 40070
    return ReturnValue(r)

# __server
def upload_contract(csvMediaId, callbackUrl, method='sync'):
    ''' update users with uploaded csv
     * method can be syncuser, replaceuser, replaceparty
    '''
    if method not in ('syncuser', 'replaceuser', 'replaceparty'):
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'method should be syncuser, replaceuser, replaceparty'})
    data = {'media_id': csvMediaId,
        'callback': {
            'url': callbackUrl,
            'token': '__server.config.token',
            'encodingaeskey': '__server.config.encodingAesKey', }}
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    @access_token
    def upload(method, accessToken=None):
        url = '%s/cgi-bin/batch/%s?access_token=%s' % \
            (COMPANY_URL, method, accessToken)
        r = requests.post(url, data=data).json()
        return ReturnValue(r)
    return upload(method)

@access_token
def get_result(jobId, accessToken=None):
    params = {
        'access_token': accessToken,
        'jobid': jobId, }
    r = requests.get('%s/cgi-bin/batch/getresult' % COMPANY_URL,
        params=params).json()
    return ReturnValue(r)



from .common import BaseController
from .mpapi.mp import templatemsgs as mpTMsg

class TemplateMsgs(BaseController):
    def set_industry(self, id1, id2):
        return self.determine_wrapper(mpTMsg.set_industry, None,
            id1, id2)
    def get_industry(self):
        return self.determine_wrapper(mpTMsg.get_industry, None)
    def add_template(self, templateId):
        return self.determine_wrapper(mpTMsg.add_template, None,
            templateId)
    def delete_templates(self, templateId):
        return self.determine_wrapper(mpTMsg.delete_templates, None,
            templateId)
    def get_templates(self):
        return self.determine_wrapper(mpTMsg.get_templates, None)
    def send(self, templateId, msgDict, toUserId):
        return self.determine_wrapper(mpTMsg.send, None,
            templateId, msgDict, toUserId)



from .common import BaseController
from .mpapi.qy import application as qyApp

class Application(BaseController):
    def get(self, agentId):
        return self.determine_wrapper(None, qyApp.get,
            agentId)
    def set(self, agentId, **kwargs):
        return self.determine_wrapper(None, qyApp.set,
            agentId, **kwargs)
    def list(self):
        return self.determine_wrapper(None, qyApp.list)



''' This package is for user managing in wechat mp
 1. What can we do?
    - set tags and everything around them
    - get all your users (of course not at a time) or get detailed information of one
    - set blacklist and everything around it

 3. I alse listed API list for you:
   - TAGS
     create_tag
     get_tags
     update_tag
     delete_tag
     get_users_of_tag
     add_users_into_tag
     delete_users_of_tag
     get_tags_of_user
   - USER INFO
     get_users
     get_user_info
     set_alias
   - BLACKLIST
     get_blacklist
     add_users_into_blacklist
     delete_users_of_blacklist
'''
import logging

from ..requests import requests
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, MUSIC, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

def create_tag(name, id=None, accessToken=None):
    ''' create_tag
     * id is for qy only
    '''
    data = encode_send_dict({'tag': {'name': name}})
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/create?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'tag' in result:
            result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def get_tags(accessToken=None):
    r = requests.get('%s/cgi-bin/tags/get?access_token=%s'
        % (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'tags' in result:
            result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def update_tag(id, name, accessToken=None):
    data = encode_send_dict({'tag': {'name': name, 'id': id}})
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/update?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def delete_tag(id, accessToken=None):
    data = encode_send_dict({'tag': {'id': id}})
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/delete?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def get_users_of_tag(id, nextOpenId='', accessToken=None):
    data = encode_send_dict({'tagid': id, 'next_openid': nextOpenId})
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/user/tag/get?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'count' in result:
            result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def add_users_into_tag(id, userIdList=None, partyList=None, accessToken=None):
    if not userIdList:
        return ReturnValue({'errcode': 40035, 'errmsg': 'must have one userId'})
    data = encode_send_dict({'openid_list': userIdList, 'tagid': id})
    if data is None:
        return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/members/batchtagging?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def delete_users_of_tag(id, userIdList=None, partyList=None, accessToken=None):
    if not userIdList: return ReturnValue({'errcode': 40035, 'errmsg': 'must have one userId'})
    data = encode_send_dict({'tagid': id, 'openid_list': userIdList})
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/members/batchuntagging?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def get_tags_of_user(userId, accessToken=None):
    data = encode_send_dict({'openid': userId})
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/tags/getidlist?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'tagid_list' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def set_alias(userId, alias, accessToken=None):
    ''' this method is for verified service mp only '''
    data = encode_send_dict({'openid': userId, 'remark': alias})
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/user/info/updateremark?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def get_user_info(userId, accessToken=None):
    ''' get info of a user or a list of users
     * userId can be a list or only one userId
    '''
    def _batch_get_user_info(userId, accessToken=None):
        data = {'user_list': [{'openid': id, 'lang': 'zh-CN'} for id in userId]}
        data = encode_send_dict(data)
        if data is None: return ReturnValue({'errcode': -10001})
        r = requests.post('%s/cgi-bin/user/info/batchget?access_token=%s'
            % (SERVER_URL, accessToken), data=data)
        def _wrap_result(result):
            result = ReturnValue(result.json())
            if 'user_info_list' in result: result['errcode'] = 0
            return result
        r._wrap_result = _wrap_result
        return result
    def _get_user_info(userId, accessToken=None):
        params = {
            'access_token': accessToken,
            'openid': userId,
            'lang': 'zh_CN', }
        r = requests.get('%s/cgi-bin/user/info' % SERVER_URL, params=params)
        def _wrap_result(result):
            result = ReturnValue(result.json())
            if 'openid' in result: result['errcode'] = 0
            return result
        r._wrap_result = _wrap_result
        return r
    if isinstance(userId, list):
        return _batch_get_user_info(userId, accessToken)
    else:
        return _get_user_info(userId, accessToken)

def get_users(nextOpenId='', departmentId=None, fetchChild=False, status=4, accessToken=None):
    ''' get users from nextOpenId
     * departmentId, fetchChild, status is for qy api
    '''
    params = {
        'access_token': accessToken,
        'next_openid': nextOpenId, }
    r = requests.get('%s/cgi-bin/user/get' % SERVER_URL, params=params)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'data' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def get_blacklist(beginOpenId='', accessToken=None):
    data = {'begin_openid': beginOpenId}
    data = encode_send_dict(data)
    r = requests.post('%s/cgi-bin/tags/members/getblacklist?access_token=%s' % 
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'data' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def add_users_into_blacklist(userId, accessToken=None):
    ''' userId can be a userId or a list of userId '''
    if not isinstance(userId, list):
        userId = [userId]
    data = {'openid_list': userId}
    data = encode_send_dict(data)
    r = requests.post('%s/cgi-bin/tags/members/batchblacklist?access_token=%s' % 
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def delete_users_of_blacklist(userId, accessToken=None):
    ''' userId can be a userId or a list of userId '''
    if not isinstance(userId, list):
        userId = [userId]
    data = {'openid_list': userId}
    data = encode_send_dict(data)
    r = requests.post('%s/cgi-bin/tags/members/batchunblacklist?access_token=%s' % 
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r



import logging, io

from tornado import gen
from ..requests import requests
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

def create_qrcode(sceneData, expire=2592000, accessToken=None):
    ''' create qrcode with specific data
     * qrcode can be permanent, if so you need to set expire to False
     * sceneData can be string or integer if it's permanent
     * but it can only be integer if it's not
    '''
    data = {'action_info': {'scene': {}}}

    try:
        expire = int(expire)
    except ValueError:
        return ReturnValue({'errcode': -10003, 'errmsg': 'expire should be int'})
    if not (isinstance(sceneData, int) or hasattr(sceneData, 'capitalize')):
        return ReturnValue({'errcode': -10003, 'errmsg':
            'sceneData should be int or string'})

    if expire:
        if not isinstance(sceneData, int):
            return ReturnValue({'errcode': -10003, 'errmsg':
                'sceneData for tmp qrcode can only be int'})
        if not 0 < expire < 2592000:
            expire = 2592000
        data['expire_seconds'] = expire
        data['action_name'] = 'QR_SCENE'
        data['action_info']['scene']['scene_id'] = sceneData
    else:
        if isinstance(sceneData, int):
            data['action_name'] = 'QR_LIMIT_SCENE'
            data['action_info']['scene']['scene_id'] = sceneData
        else:
            data['action_name'] = 'QR_LIMIT_STR_SCENE'
            data['action_info']['scene']['scene_str'] = sceneData

    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/qrcode/create?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'ticket' in result:
            result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def download_qrcode(ticket):
    if COROUTINE:
        @gen.coroutine
        def _download_qrcode(ticket):
            params = {'ticket': ticket}
            r = yield requests.get('https://mp.weixin.qq.com/cgi-bin/showqrcode',
                params=params, stream=True)
            if 'application/json' in r.headers['Content-Type']:
                r = ReturnValue(r.json())
            else:
                tempStorage = io.BytesIO()
                for block in r.iter_content(1024):
                    tempStorage.write(block)
                r = ReturnValue({'File': tempStorage, 'errcode': 0})
            raise gen.Return(r)
    else:
        def _download_qrcode(ticket):
            params = {'ticket': ticket}
            r = requests.get('https://mp.weixin.qq.com/cgi-bin/showqrcode',
                params=params, stream=True)
            if 'application/json' in r.headers['Content-Type']:
                r = ReturnValue(r.json())
            else:
                tempStorage = io.BytesIO()
                for block in r.iter_content(1024):
                    tempStorage.write(block)
                r = ReturnValue({'File': tempStorage, 'errcode': 0})
            return r
    return _download_qrcode()

def long_url_to_short(url, accessToken=None):
    data = {'action': 'long2short', 'long_url': url}
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/shorturl?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r



from .common import BaseController
from .mpapi.mp import menu as mpMenu
from .mpapi.qy import menu as qyMenu

class Menu(BaseController):
    def create(self, menuDict, agentId=None):
        return self.determine_wrapper(mpMenu.create, qyMenu.create,
            menuDict, agentId)
    def get(self, agentId=None):
        return self.determine_wrapper(mpMenu.get, qyMenu.get,
            agentId)
    def delete(self, agentId=None):
        return self.determine_wrapper(mpMenu.delete, qyMenu.delete,
            agentId)
    def addconditional(self, menuDict):
        return self.determine_wrapper(mpMenu.addconditional, None,
            menuDict)
    def delconditional(self, menuId):
        return self.determine_wrapper(mpMenu.delconditional, None,
            menuId)
    def trymatch(self, userId):
        return self.determine_wrapper(mpMenu.trymatch, None,
            userId)
    def get_current_selfmenu_info(self):
        return self.determine_wrapper(mpMenu.get_current_selfmenu_info, None)



import functools, logging, time, threading, traceback
from datetime import datetime, timedelta

from requests import get
from requests.models import Response
from requests.sessions import session
from tornado import gen

from ..requests import requests
from itchatmp.config import COROUTINE
from itchatmp.utils import retry, CoreMixin
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

class TokenClass(CoreMixin):
    def __init__(self, core, tokenUrl, appIdFn):
        super(TokenClass, self).__init__(core)
        self.tokenUrl, self.appIdFn = tokenUrl, appIdFn
        self._session = session()
        self._session.verify = False
        self._autoMaintain = False
        self._tokenFunction = self.token_function_producer()
        self._syncTokenFunction = self.token_function_producer(True)
        self._accessTokenFunction = self.access_token_producer()
        self._syncAccessTokenFunction = self.access_token_producer(True)
    def token_function_producer(self, forceSync=False):
        ''' function to update access token
         * auto-maintain begins when this function is called for the first time
           - of course, if isWsgi or haven't started, auto-maintain will not start
         * If auto-maintain failed, it will restart tomorrow 0004h:30
         * will return ReturnValue object (equals to True) if success
         * will return ReturnValue object (equals to False) if fail
         * ReturnValue object contains why update failed
        '''
        if COROUTINE and not forceSync:
            @gen.coroutine
            def _update_access_token():
                url = self.tokenUrl % (self.appIdFn(self.core), self.core.config.appSecret)
                r = (yield requests.get(url)).json()
                if 'access_token' in r:
                    self.core.atStorage.store_access_token(
                        r['access_token'], int(time.time()) + r['expires_in'])
                    r['errcode'] = 0
                    r = ReturnValue(r)
                else:
                    r = ReturnValue(r)
                    logger.debug('Failed to get token: %s' %r)
                if not self._autoMaintain and not self.core.isWsgi:
                    self._autoMaintain = True
                    self.maintain_access_token(r)
                raise gen.Return(r)
            return _update_access_token
        else:
            def _update_access_token():
                url = self.tokenUrl % (self.appIdFn(self.core), self.core.config.appSecret)
                r = self._session.get(url).json()
                if 'access_token' in r:
                    self.core.atStorage.store_access_token(
                        r['access_token'], int(time.time()) + r['expires_in'])
                    r['errcode'] = 0
                    r = ReturnValue(r)
                else:
                    r = ReturnValue(r)
                    logger.debug('Failed to get token: %s' %r)
                if not self._autoMaintain and not self.core.isWsgi:
                    self._autoMaintain = True
                    self.maintain_access_token(r)
                return r
            return _update_access_token
    def maintain_thread(self, firstCallResult=None):
        ''' thread to update token and
        register next update event into event loop '''
        r = firstCallResult or self._syncTokenFunction()
        if not r:
            self.core.ioLoop.call_later(
                (datetime.replace(datetime.now() + timedelta(days=1), 
                hour=0, minute=5, second=0) - datetime.now()).seconds,
                self.maintain_access_token, None)
        else:
            self.core.ioLoop.call_later(r['expires_in'] - 30,
                self.maintain_access_token, None)
    def maintain_access_token(self, firstCallResult=None):
        t = threading.Thread(target=auto_maintain_thread,
            args=(firstCallResult,))
        t.setDaemon(True)
        t.start()
    def access_token_producer(self, forceSync=False):
        ''' wrapper for functions need accessToken
            accessToken should be a key argument of the wrapped fn
            ..code:: python
                 @access_token
                 def fn(a, b, c, accessToken=False):
                     pass
            if accessToken is not successfully fetched, wrapped fn will return why
            There's a very tricky thing about this decorator:
                You need to return a requests.models.Response object
                And set _wrap_result of the object to format result
        '''
        if COROUTINE and not forceSync:
            def _access_token(fn):
                @gen.coroutine
                @functools.wraps(fn)
                def __access_token(*args, **kwargs):
                    accessToken, expireTime = self.core.atStorage.get_access_token()
                    if expireTime < time.time():
                        updateResult = yield self.update_access_token()
                        if not updateResult:
                            raise gen.Return(updateResult)
                        accessToken, expireTime = self.core.atStorage.get_access_token()
                    kwargs['accessToken'] = accessToken
                    future = fn(*args, **kwargs)
                    if gen.is_future(future):
                        r = yield future
                    else:
                        r = future
                    try:
                        errcode = r.json().get('errcode')
                        isTokenTimeout = errcode == 40014
                    except:
                        isTokenTimeout = False
                    if isinstance(r, Response) and isTokenTimeout:
                        updateResult = yield self.update_access_token()
                        if not updateResult:
                            raise gen.Return(updateResult)
                        accessToken, expireTime = self.core.atStorage.get_access_token()
                        kwargs['accessToken'] = accessToken
                        future = fn(*args, **kwargs)
                        r = yield future
                    wrap_fn = getattr(future, '_wrap_result', None)
                    if wrap_fn is not None:
                        try:
                            r = wrap_fn(r)
                        except:
                            r = ReturnValue({'errcode': -10005, 'errmsg': traceback.format_exc()})
                    raise gen.Return(r)
                return __access_token
            return _access_token
        else:
            def _access_token(fn):
                @functools.wraps(fn)
                def __access_token(*args, **kwargs):
                    accessToken, expireTime = self.core.atStorage.get_access_token()
                    if expireTime < time.time():
                        updateResult = self.update_access_token()
                        if not updateResult:
                            return updateResult
                        accessToken, expireTime = self.core.atStorage.get_access_token()
                    kwargs['accessToken'] = accessToken
                    r = fn(*args, **kwargs)
                    try:
                        errcode = r.json().get('errcode')
                        isTokenTimeout = errcode == 40014
                    except:
                        isTokenTimeout = False
                    if isinstance(r, Response) and isTokenTimeout:
                        updateResult = self.update_access_token()
                        if not updateResult:
                            return updateResult
                        accessToken, expireTime = self.core.atStorage.get_access_token()
                        kwargs['accessToken'] = accessToken
                        r = fn(*args, **kwargs)
                    wrap_fn = getattr(r, '_wrap_result', None)
                    if wrap_fn is not None:
                        try:
                            r = wrap_fn(r)
                        except:
                            r = ReturnValue({'errcode': -10005, 'errmsg': traceback.format_exc()})
                    return r
                return __access_token
            return _access_token
    def update_access_token(self):
        return self._tokenFunction()
    def access_token(self, fn):
        return self._accessTokenFunction(fn)

class ServerListClass(CoreMixin):
    def __init__(self, core, tokenClass, serverUrl):
        super(ServerListClass, self).__init__(core)
        self.tokenClass = tokenClass
        self.serverUrl = serverUrl
        self._serverList = None
        self._serverIpFn = self.get_server_ip_producer()
        self._syncServerIpFn = self.get_server_ip_producer(True)
    def set_server_list(self):
        self._serverList = []
        serverList, fetchTime = self.core.atStorage.get_server_list()
        if fetchTime < time.mktime(datetime.replace(datetime.now(),
            hour=0, minute=0, second=0).timetuple()):
            r = self._serverIpFn()
            if not r:
                logger.debug(r)
            else:
                self._serverList = r.get('ip_list', [])
                self.core.atStorage.store_server_list(self._serverList, time.time())
        else:
            self._serverList = serverList
    def get_server_ip_producer(self, forceSync=False):
        if COROUTINE and not forceSync:
            access_token = self.tokenClass._accessTokenFunction
            requests_get = requests.get
        else:
            access_token = self.tokenClass._syncAccessTokenFunction
            requests_get = get
        @access_token
        def _get_server_ip(accessToken=None):
            url = self.serverUrl % accessToken
            r = requests_get(url)
            def _wrap_result(result):
                result = ReturnValue(result.json())
                if 'ip_list' in result:
                    result['errcode'] = 0
                    for i, v in enumerate(result['ip_list']):
                        result['ip_list'][i] = v[:v.rfind('/')]
                return result
            r._wrap_result = _wrap_result
            return r
        return _get_server_ip
    def get_server_ip(self):
        return self._serverIpFn()
    def filter_request(self, request):
        if self._serverList is None:
            t = threading.Thread(target=self.set_server_list)
            t.setDaemon = True
            t.start()
            def clear_server_list():
                self._serverList = None
            self.core.ioLoop.call_later(
                (datetime.replace(datetime.now() + timedelta(days=1), 
                hour=0, minute=5, second=0) - datetime.now()).seconds,
                clear_server_list)
        if not self._serverList:
            logger.debug('Server list is loading, so ignore verifying once.')
            return True
        return request.remote_ip in self._serverList
